@@ -11,21 +11,29 @@ module MIME
   # == Usage
   #  require 'mime/types'
   #
-  #  plaintext = MIME::Types['text/plain']
-  #  print plaintext.media_type           # => 'text'
-  #  print plaintext.sub_type             # => 'plain'
+  #  plaintext = MIME::Types['text/plain'].first
+  #  # returns [text/plain, text/plain]
+  #  text      = plaintext.first
+  #  print text.media_type           # => 'text'
+  #  print text.sub_type             # => 'plain'
   #
-  #  puts plaintext.extensions.join(" ")  # => 'asc txt c cc h hh cpp'
+  #  puts text.extensions.join(" ")  # => 'asc txt c cc h hh cpp'
   #
-  #  puts plaintext.encoding              # => 8bit
-  #  puts plaintext.binary?               # => false
-  #  puts plaintext.ascii?                # => true
-  #  puts plaintext == 'text/plain'       # => true
+  #  puts text.encoding              # => 8bit
+  #  puts text.binary?               # => false
+  #  puts text.ascii?                # => true
+  #  puts text == 'text/plain'       # => true
   #  puts MIME::Type.simplified('x-appl/x-zip') # => 'appl/zip'
   #
+  #  puts MIME::Types.any? { |type|
+  #    type.content_type == 'text/plain'
+  #  }                               # => true
+  #  puts MIME::Types.all?(&:registered?)
+  #                                  # => false
+  #
   class Type
     # The released version of Ruby MIME::Types
-    VERSION = '1.19'
+    VERSION = '1.25'
 
     include Comparable
 
@@ -74,39 +82,35 @@ module MIME
     #
     # 1. self.simplified <=> other.simplified (ensures that we
     #    don't try to compare different types)
-    # 2. IANA-registered definitions > other definitions.
-    # 3. Generic definitions > platform definitions.
-    # 3. Complete definitions > incomplete definitions.
-    # 4. Current definitions > obsolete definitions.
-    # 5. Obselete with use-instead references > obsolete without.
+    # 2. IANA-registered definitions < other definitions.
+    # 3. Generic definitions < platform definitions.
+    # 3. Complete definitions < incomplete definitions.
+    # 4. Current definitions < obsolete definitions.
+    # 5. Obselete with use-instead references < obsolete without.
     # 6. Obsolete use-instead definitions are compared.
     def priority_compare(other)
       pc = simplified <=> other.simplified
 
-      if pc.zero? and registered? != other.registered?
-        pc = registered? ? -1 : 1
-      end
-
-      if pc.zero? and platform? != other.platform?
-        pc = platform? ? 1 : -1
-      end
-
-      if pc.zero? and complete? != other.complete?
-        pc = complete? ? -1 : 1
-      end
-
-      if pc.zero? and obsolete? != other.obsolete?
-        pc = obsolete? ? 1 : -1
-      end
-
-      if pc.zero? and obsolete? and (use_instead != other.use_instead)
-        pc = if use_instead.nil?
-               -1
-             elsif other.use_instead.nil?
-               1
-             else
-               use_instead <=> other.use_instead
+      if pc.zero?
+        pc = if registered? != other.registered?
+               registered? ? -1 : 1 # registered < unregistered
+             elsif platform? != other.platform?
+               platform? ? 1 : -1 # generic < platform
+             elsif complete? != other.complete?
+               pc = complete? ? -1 : 1 # complete < incomplete
+             elsif obsolete? != other.obsolete?
+               pc = obsolete? ? 1 : -1 # current < obsolete
              end
+
+        if pc.zero? and obsolete? and (use_instead != other.use_instead)
+          pc = if use_instead.nil?
+                 -1
+               elsif other.use_instead.nil?
+                 1
+               else
+                 use_instead <=> other.use_instead
+               end
+        end
       end
 
       pc
@@ -270,7 +274,7 @@ module MIME
           DRAFT_URL % $1
         when %r{^LTSW$}
           LTSW_URL % @media_type
-        when %r<^\{([^=]+)=([^\]]+)\}>
+        when %r{^\{([^=]+)=([^\}]+)\}}
           [$1, $2]
         when %r{^\[([^=]+)=([^\]]+)\]}
           [$1, CONTACT_URL % $2]
@@ -317,23 +321,17 @@ module MIME
       #   end
       def from_array(*args) #:yields MIME::Type.new:
         # Dereferences the array one level, if necessary.
-        args = args[0] if args[0].kind_of?(Array)
+        args = args.first if args.first.kind_of? Array
 
-        if args.size.between?(1, 8)
-          m = MIME::Type.new(args[0]) do |t|
-            t.extensions  = args[1] if args.size > 1
-            t.encoding    = args[2] if args.size > 2
-            t.system      = args[3] if args.size > 3
-            t.obsolete    = args[4] if args.size > 4
-            t.docs        = args[5] if args.size > 5
-            t.url         = args[6] if args.size > 6
-            t.registered  = args[7] if args.size > 7
-          end
-          yield m if block_given?
-        else
+        unless args.size.between?(1, 8)
           raise ArgumentError, "Array provided must contain between one and eight elements."
         end
-        m
+
+        MIME::Type.new(args.shift) do |t|
+          t.extensions, t.encoding, t.system, t.obsolete, t.docs, t.url,
+            t.registered = *args
+          yield t if block_given?
+        end
       end
 
       # Creates a MIME::Type from a hash. Keys are case-insensitive,
@@ -364,7 +362,7 @@ module MIME
           type[k.to_s.tr('A-Z', 'a-z').gsub(/-/, '_').to_sym] = v
         end
 
-        m = MIME::Type.new(type[:content_type]) do |t|
+        MIME::Type.new(type[:content_type]) do |t|
           t.extensions  = type[:extensions]
           t.encoding    = type[:content_transfer_encoding]
           t.system      = type[:system]
@@ -372,10 +370,9 @@ module MIME
           t.docs        = type[:docs]
           t.url         = type[:url]
           t.registered  = type[:registered]
-        end
 
-        yield m if block_given?
-        m
+          yield t if block_given?
+        end
       end
 
       # Essentially a copy constructor.
@@ -390,7 +387,7 @@ module MIME
       #     t.encoding    = plaintext.encoding.dup
       #   end
       def from_mime_type(mime_type) #:yields the new MIME::Type:
-        m = MIME::Type.new(mime_type.content_type.dup) do |t|
+        MIME::Type.new(mime_type.content_type.dup) do |t|
           t.extensions = mime_type.extensions.map { |e| e.dup }
           t.url = mime_type.url && mime_type.url.map { |e| e.dup }
 
@@ -402,9 +399,8 @@ module MIME
 
           mime_type.docs && t.docs = mime_type.docs.dup
 
+          yield t if block_given?
         end
-
-        yield m if block_given?
       end
     end
 
@@ -561,32 +557,39 @@ module MIME
   # ftp://ftp.iana.org/assignments/media-types with some unofficial types
   # added from the the collection at
   # http://www.ltsw.se/knbase/internet/mime.htp
-  #
-  # This is originally based on Perl MIME::Types by Mark Overmeer.
-  #
-  # = Author
-  # Copyright:: Copyright 2002–2012 by Austin Ziegler
-  #             <austin@rubyforge.org>
-  # Version::   1.19
-  # Based On::  Perl
-  #             MIME::Types[http://search.cpan.org/author/MARKOV/MIME-Types-1.27/MIME/Types.pm],
-  #             Copyright 2001–2009 by Mark Overmeer
-  #             <mimetypes@overmeer.net>.
-  # Licence::   Ruby's, Perl Artistic, or GPL version 2 (or later)
-  # See Also::  http://www.iana.org/assignments/media-types/
-  #             http://www.ltsw.se/knbase/internet/mime.htp
-  #
   class Types
     # The released version of Ruby MIME::Types
-    VERSION = MIME::Type::VERSION
+    VERSION      = MIME::Type::VERSION
+    DATA_VERSION = (VERSION.to_f * 100).to_i
 
-      # The data version.
+    # The data version.
     attr_reader :data_version
 
-    def initialize(data_version = nil)
-      @type_variants    = Hash.new { |h, k| h[k] = [] }
-      @extension_index  = Hash.new { |h, k| h[k] = [] }
-      @data_version = data_version
+    class HashWithArrayDefault < Hash # :nodoc:
+      def initialize
+        super { |h, k| h[k] = [] }
+      end
+
+      def marshal_dump
+        {}.merge(self)
+      end
+
+      def marshal_load(hash)
+        self.merge!(hash)
+      end
+    end
+
+    class CacheContainer # :nodoc:
+      attr_reader :version, :data
+      def initialize(version, data)
+        @version, @data = version, data
+      end
+    end
+
+    def initialize(data_version = DATA_VERSION)
+      @type_variants    = HashWithArrayDefault.new
+      @extension_index  = HashWithArrayDefault.new
+      @data_version     = data_version
     end
 
     def add_type_variant(mime_type) #:nodoc:
@@ -601,7 +604,18 @@ module MIME
       @type_variants.values.flatten
     end
 
-    @__types__ = self.new(VERSION)
+    # Returns the number of known types. A shortcut of MIME::Types[//].size.
+    # (Keep in mind that this is memory intensive, cache the result to spare
+    # resources)
+    def count
+      defined_types.size
+    end
+
+    def each
+     defined_types.each { |t| yield t }
+    end
+
+    @__types__  = nil
 
     # Returns a list of MIME::Type objects, which may be empty. The optional
     # flag parameters are :complete (finds only complete MIME::Type objects)
@@ -630,22 +644,16 @@ module MIME
     #   6. Obsolete definitions use-instead clauses are compared.
     #   7. Sort on name.
     def [](type_id, flags = {})
-      if type_id.kind_of?(Regexp)
-        matches = []
-        @type_variants.each_key do |k|
-          matches << @type_variants[k] if k =~ type_id
-        end
-        matches.flatten!
-      elsif type_id.kind_of?(MIME::Type)
-        matches = [type_id]
-      else
-        matches = @type_variants[MIME::Type.simplified(type_id)]
-      end
+      matches = case type_id
+                when MIME::Type
+                  @type_variants[type_id.simplified]
+                when Regexp
+                  match(type_id)
+                else
+                  @type_variants[MIME::Type.simplified(type_id)]
+                end
 
-      matches.delete_if { |e| not e.complete? } if flags[:complete]
-      matches.delete_if { |e| not e.platform? } if flags[:platform]
-
-      matches.sort { |a, b| a.priority_compare(b) }
+      prune_matches(matches, flags).sort { |a, b| a.priority_compare(b) }
     end
 
     # Return the list of MIME::Types which belongs to the file based on its
@@ -683,7 +691,7 @@ module MIME
         else
           if @type_variants.include?(mime_type.simplified)
             if @type_variants[mime_type.simplified].include?(mime_type)
-              warn "Type #{mime_type} already registered as a variant of #{mime_type.simplified}." unless defined? MIME::Types::STARTUP
+              warn "Type #{mime_type} already registered as a variant of #{mime_type.simplified}." unless defined? MIME::Types::LOAD
             end
           end
           add_type_variant(mime_type)
@@ -692,30 +700,47 @@ module MIME
       end
     end
 
+    private
+    def prune_matches(matches, flags)
+      matches.delete_if { |e| not e.complete? } if flags[:complete]
+      matches.delete_if { |e| not e.platform? } if flags[:platform]
+      matches
+    end
+
+    def match(pattern)
+      matches = @type_variants.select { |k, v| k =~ pattern }
+      if matches.respond_to? :values
+        matches.values.flatten
+      else
+        matches.map { |m| m.last }.flatten
+      end
+    end
+
     class << self
       def add_type_variant(mime_type) #:nodoc:
-        @__types__.add_type_variant(mime_type)
+        __types__.add_type_variant(mime_type)
       end
 
       def index_extensions(mime_type) #:nodoc:
-        @__types__.index_extensions(mime_type)
+        __types__.index_extensions(mime_type)
       end
 
       # The regular expression used to match a file-based MIME type
       # definition.
       TEXT_FORMAT_RE = %r{
-        ^
+        \A
         \s*
         ([*])?                                 # 0: Unregistered?
         (!)?                                   # 1: Obsolete?
         (?:(\w+):)?                            # 2: Platform marker
-        #{MIME::Type::MEDIA_TYPE_RE}           # 3,4: Media type
+        #{MIME::Type::MEDIA_TYPE_RE}?          # 3,4: Media type
         (?:\s+@([^\s]+))?                      # 5: Extensions
         (?:\s+:(#{MIME::Type::ENCODING_RE}))?  # 6: Encoding
         (?:\s+'(.+))?                          # 7: URL list
         (?:\s+=(.+))?                          # 8: Documentation
+        (?:\s*([#].*)?)?
         \s*
-        $
+        \z
       }x
 
       # Build the type list from a file in the format:
@@ -756,14 +781,14 @@ module MIME
       # be provided.
       def load_from_file(filename) #:nodoc:
         if defined? ::Encoding
-          data = File.open(filename, 'r:UTF-8') { |f| f.read }
+          data = File.open(filename, 'r:UTF-8:-') { |f| f.read }
         else
           data = File.open(filename) { |f| f.read }
         end
         data = data.split($/)
         mime = MIME::Types.new
         data.each_with_index { |line, index|
-          item = line.chomp.strip.gsub(%r{#.*}o, '')
+          item = line.chomp.strip
           next if item.empty?
 
           begin
@@ -775,7 +800,17 @@ module MIME
           end
 
           unregistered, obsolete, platform, mediatype, subtype, extensions,
-            encoding, urls, docs = *m
+            encoding, urls, docs, comment = *m
+
+          if mediatype.nil?
+            if comment.nil?
+              puts "#{filename}:#{index}: Parsing error in MIME type definitions."
+              puts "=> #{line}"
+              raise RuntimeError
+            end
+
+            next
+          end
 
           extensions &&= extensions.split(/,/)
           urls &&= urls.split(/,/)
@@ -810,7 +845,17 @@ module MIME
       #     puts t.to_a.join(", ")
       #   end
       def [](type_id, flags = {})
-        @__types__[type_id, flags]
+        __types__[type_id, flags]
+      end
+
+      include Enumerable
+
+      def count
+        __types__.count
+      end
+
+      def each
+        __types__.each {|t| yield t }
       end
 
       # Return the list of MIME::Types which belongs to the file based on
@@ -824,12 +869,12 @@ module MIME
       #   puts "MIME::Types.type_for('citydesk.gif')
       #     => [image/gif]
       def type_for(filename, platform = false)
-        @__types__.type_for(filename, platform)
+        __types__.type_for(filename, platform)
       end
 
       # A synonym for MIME::Types.type_for
       def of(filename, platform = false)
-        @__types__.type_for(filename, platform)
+        __types__.type_for(filename, platform)
       end
 
       # Add one or more MIME::Type objects to the set of known types. Each
@@ -839,14 +884,82 @@ module MIME
       # <strong>Please inform the maintainer of this module when registered
       # types are missing.</strong>
       def add(*types)
-        @__types__.add(*types)
+        __types__.add(*types)
+      end
+
+      # Returns the currently defined cache file, if any.
+      def cache_file
+        ENV['RUBY_MIME_TYPES_CACHE']
+      end
+
+      private
+      def load_mime_types_from_cache
+        load_mime_types_from_cache! if cache_file
+      end
+
+      def load_mime_types_from_cache!
+        raise ArgumentError, "No RUBY_MIME_TYPES_CACHE set." unless cache_file
+        return false unless File.exists? cache_file
+
+        begin
+          data      = File.read(cache_file)
+          container = Marshal.load(data)
+
+          if container.version == VERSION
+            @__types__ = Marshal.load(container.data)
+            true
+          else
+            false
+          end
+        rescue => e
+          warn "Could not load MIME::Types cache: #{e}"
+          false
+        end
+      end
+
+      def write_mime_types_to_cache
+        write_mime_types_to_cache! if cache_file
+      end
+
+      def write_mime_types_to_cache!
+        raise ArgumentError, "No RUBY_MIME_TYPES_CACHE set." unless cache_file
+
+        File.open(cache_file, 'w') do |f|
+          cache = MIME::Types::CacheContainer.new(VERSION,
+                                                  Marshal.dump(__types__))
+          f.write Marshal.dump(cache)
+        end
+
+        true
+      end
+
+      def load_and_parse_mime_types
+        const_set(:LOAD, true) unless $DEBUG
+        Dir[File.join(File.dirname(__FILE__), 'types', '*')].sort.each { |f|
+          add(load_from_file(f))
+        }
+        remove_const :LOAD if defined? LOAD
+      end
+
+      def lazy_load?
+        (lazy = ENV['RUBY_MIME_TYPES_LAZY_LOAD']) && (lazy != 'false')
+      end
+
+      def __types__
+        load_mime_types unless @__types__
+        @__types__
+      end
+
+      def load_mime_types
+        @__types__ = new(VERSION)
+        unless load_mime_types_from_cache
+          load_and_parse_mime_types
+          write_mime_types_to_cache
+        end
       end
     end
 
-    files = Dir[File.join(File.dirname(__FILE__), 'types', '*')]
-    MIME::Types::STARTUP = true unless $DEBUG
-    files.sort.each { |file| add load_from_file(file) }
-    remove_const :STARTUP if defined? STARTUP
+    load_mime_types unless lazy_load?
   end
 end
 
